<?php
/**
 * @file
 * Drupal RestMini Client module.
 */

/**
 * Drush REST client.
 *
 * drush rcr GET 'http://ser.ver' 'endpoint' '{\"path\":[\"path-arg\"]}'
 *
 * @return array
 */
function restmini_client_drush_command() {
  // This module must be enabled, otherwise classes RestMiniClient and RestMini
  // aren't included.
  // The function_exists() check prevents error during unorthodox 'preload
  // drush' procedure. We could instead include the classes via include_once,
  // but that would present a range of other risks; and actual support for such
  // unorthodox procedure cannot be a priority.
  if (!function_exists('module_exists') || !module_exists('restmini_client')) {
    return array();
  }

  $options_supported = array();
  $a = RestMiniClient::optionsSupported();
  foreach ($a as $name) {
    $options_supported[str_replace('_', '-', $name)] = ' ';
  }
  unset($a);

  return array(
    'restmini-client' => array(
      'description' => 'Send HTTP(S) REST request via RESTmini Client.',
      'arguments' => array(
        'method' => 'HEAD | GET | POST | PUT | DELETE',
        'server' => 'protocol://domain.tld',
        'endpoint' => 'path | /base/route/end-point | /endpoint.php | /dir/endpoint.aspx?arg=val',
        'parameters' => '(optional) JSON-encoded list of path, query and/or body arguments',
      ),
      'options' => $options_supported,
      'examples' => array(
        "drush restmini-client GET 'http://ser.ver' 'endpoint'" => ' ',
        "drush restmini-client POST 'http://ser.ver' 'endpoint' '{\"path\":[\"path-arg-1\",\"path-arg-2\"],\"body\":{\"body-param\":\"body-arg\"}}'" => ' ',
        "drush rcr GET 'https://ser.ver' 'endpoint' --ssl-verify=0" => ' ',
        "drush rcr GET http://ser.ver service/restmini-example/example/thingie" => ' ',
        "drush rcr GET http://ser.ver service/restmini-example/example/thingie '{\"path\":[3]}'" => ' ',
      ),
      'aliases' => array('rcr'),
      'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_FULL,
    )
  );
}

/**
 * @param string $method
 *   HEAD | GET | POST | PUT | DELETE.
 * @param string $server
 *   protocol://domain.tld.
 * @param string $endpoint
 *   path | /base/route/end-point | /endpoint.php | /dir/endpoint.aspx?arg=val.
 *   Default: empty.
 * @param string $parameters
 *   JSON-encoded list of path, query and/or body arguments.
 *   Default: empty.
 */
function drush_restmini_client($method, $server, $endpoint = '', $parameters = '') {
  // Check that we are in CLI mode.
  if (!drupal_is_cli()) {
    drush_log('Function ' . __FUNCTION__ . ' is only callable from drush/cli.', 'error');
    return;
  }
  // Check that this module is enabled.
  if (!module_exists('restmini_client')) {
    drush_log('This restmini_client module isnt enabled.', 'error');
    return;
  }

  // Check/get args.
  if (!$method) {
    drush_log('Argument method is empty.', 'error');
    return;
  }
  $method = strtoupper($method);

  if (!$server) {
    drush_log('Argument server is empty.', 'error');
    return;
  }

  $params = array(
    'path' => array(),
    'query' => array(),
    'body' => array(),
  );
  if ($parameters && $parameters !== '{}') {
    $parameters = drupal_json_decode($parameters);
    if (!$parameters) {
      drush_log('Argument parameters is not valid JSON.', 'error');
      return;
    }
    if (!is_array($parameters)) {
      drush_log('Argument parameters must be a JSON-encoded list of path, query and/or body arguments, or empty.', 'error');
      return;
    }
    if (array_key_exists('path', $parameters)) {
      if ($parameters['path']) {
        $params['path'] = $parameters['path'];
      }
      unset($parameters['path']);
    }
    if (array_key_exists('query', $parameters)) {
      if ($parameters['query']) {
        $params['query'] = $parameters['query'];
      }
      unset($parameters['query']);
    }
    // Backwards compatibility; 'query' params used to be called 'get' params.
    elseif (array_key_exists('get', $parameters)) {
      if ($parameters['get']) {
        $params['query'] = $parameters['get'];
      }
      unset($parameters['get']);
    }
    if (array_key_exists('body', $parameters)) {
      if ($parameters['body']) {
        $params['body'] = $parameters['body'];
      }
      unset($parameters['body']);
    }
    // Backwards compatibility; 'body' params used to be called 'post' params.
    elseif (array_key_exists('post', $parameters)) {
      if ($parameters['post']) {
        $params['body'] = $parameters['post'];
      }
      unset($parameters['post']);
    }
    if ($parameters) {
      drush_log('Argument parameters sub-list(s) ' . join(', ', array_keys($parameters)) . ' are not valid sub-lists, use path, query and/or body.', 'error');
      return;
    }
  }

  // Resolve options.
  $options = array();
  $options_supported = RestMiniClient::optionsSupported();
  foreach ($options_supported as $name) {
    $value = drush_get_option(str_replace('_', '-', $name));
    if ($value !== NULL) {
      $options[$name] = $value;
    }
  }
  unset($options_supported);

  $client = RestMiniClient::make($server, $endpoint, $options)->request($method, $params['path'], $params['query'], $params['body']);
  $info = $client->info();
  if ($info['error']) {
    if (module_exists('inspect')) {
      inspect(
        array(
          'info' => $info,
          'raw result' => $client->raw(),
        ),
        array(
          'category' => 'restmini_client',
          'message' => 'REST request failed',
          'severity' => WATCHDOG_ERROR,
        )
      );
    }
    else {
      watchdog(
        'restmini_client',
        'REST request failed',
        array(),
        WATCHDOG_ERROR
      );
    }
    drush_log($info['status'] . ' - request failed.', 'error');
  }
  else {
    drush_log($info['status'] . ' - request succeeded.', 'success');
  }
  $filename = $method . '__' . preg_replace('/[^a-zA-Z\d_\-\.]/', '--', preg_replace('/^(https?):\/\//', '$1_', $info['url'])) . '.txt';
  file_put_contents($_SERVER['PWD'] . '/' . $filename, $client->raw());
  drush_log('Response filed to ' . $filename, 'status');
}
